"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const path_1 = __importDefault(require("path"));
const cookie_version_matcher_1 = require("./cookie-version-matcher");
class Api {
    constructor(config) {
        this.handler = {};
        this.config = config;
        if (this.config.versionMatcher) {
            this.versionMatcher = new cookie_version_matcher_1.CookieVersionMatcher(this.config.versionMatcher);
        }
        this.prepareHandlers();
    }
    /**
     * Registers API endpoints /{API_PREFIX}/{APINAME}
     * @param {Server} app
     */
    registerEndpoints(app) {
        app.addUse(`/${config_1.API_ROUTE_PREFIX}/${this.config.name}`, (req, res, next) => {
            const requestVersion = this.detectVersion(req.cookies);
            req.headers["originalurl"] = req.url;
            req.headers["originalpath"] = req.path;
            req.url = `/${requestVersion}${req.url}`;
            next();
        });
        Object.keys(this.config.versions).forEach(version => {
            const apiHandler = this.config.versions[version];
            apiHandler.endpoints.forEach(endpoint => {
                app.addRoute(`/${config_1.API_ROUTE_PREFIX}/${this.config.name}/${version}${endpoint.path}`, endpoint.method, this.handler[version][endpoint.controller], endpoint.middlewares);
            });
        });
    }
    /**
     * Resolves version handlers based on configuration
     */
    prepareHandlers() {
        Object.keys(this.config.versions).forEach(version => {
            const configurationHandler = this.config.versions[version].handler;
            if (configurationHandler) {
                this.handler[version] = configurationHandler;
            }
            else {
                this.handler[version] = require(path_1.default.join(process.cwd(), `/src/api/`, this.config.name, version));
            }
        });
    }
    detectVersion(cookie) {
        const cookieKey = this.config.testCookie;
        const cookieVersion = cookie[cookieKey] && this.config.versions[cookie[cookieKey]] ? cookie[cookieKey] : null;
        if (cookieVersion)
            return cookieVersion;
        const matcherVersion = this.versionMatcher ? this.versionMatcher.match(cookie) : null;
        if (matcherVersion && this.config.versions[matcherVersion])
            return matcherVersion;
        return this.config.liveVersion;
    }
}
exports.Api = Api;
//# sourceMappingURL=api.js.map