"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var HttpClient_1;
Object.defineProperty(exports, "__esModule", { value: true });
"use strict";
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const inversify_1 = require("inversify");
const request_1 = __importDefault(require("request"));
const config_1 = require("./config");
const puzzle_warden_1 = __importDefault(require("puzzle-warden"));
const AGENT_CONFIGURATION = {
    keepAlive: true,
    maxSockets: config_1.PUZZLE_MAX_SOCKETS,
    keepAliveMsecs: config_1.KEEP_ALIVE_MSECS
};
exports.httpAgent = new http_1.default.Agent(AGENT_CONFIGURATION);
exports.httpsAgent = new https_1.default.Agent(AGENT_CONFIGURATION);
let HttpClient = HttpClient_1 = class HttpClient {
    constructor() {
        this.httpAgent = exports.httpAgent;
        this.httpsAgent = exports.httpsAgent;
    }
    init(clientName, options) {
        this.defaultOptions = Object.assign({ encoding: 'utf8', gzip: true }, options, { headers: {
                'user-agent': clientName || 'PuzzleJs Http Client'
            } });
        HttpClient_1.httpClient = request_1.default.defaults(Object.assign({}, this.defaultOptions, { agent: exports.httpAgent }));
        HttpClient_1.httpsClient = request_1.default.defaults(Object.assign({}, this.defaultOptions, { agent: exports.httpsAgent }));
    }
    get(requestUrl, fragmentName, options) {
        if (!HttpClient_1.httpClient && !HttpClient_1.httpsClient) {
            console.error('Creating new agent for pool');
            this.init('PuzzleJs Default Client');
        }
        const client = requestUrl.startsWith('https') ? HttpClient_1.httpsClient : HttpClient_1.httpClient;
        return new Promise(function (resolve, reject) {
            const requestOptions = Object.assign({ url: requestUrl, method: 'get' }, options);
            const cb = (err, response, data) => {
                if (err)
                    reject(err);
                resolve({
                    response,
                    data
                });
            };
            if (puzzle_warden_1.default.isRouteRegistered(fragmentName)) {
                puzzle_warden_1.default.request(fragmentName, requestOptions, cb);
            }
            else {
                client(requestOptions, cb);
            }
        });
    }
    post(requestUrl, fragmentName, data, options) {
        if (!HttpClient_1.httpClient && !HttpClient_1.httpsClient)
            this.init('PuzzleJs Default Client');
        const client = requestUrl.startsWith('https') ? HttpClient_1.httpsClient : HttpClient_1.httpClient;
        return new Promise(function (resolve, reject) {
            client
                .post(Object.assign({ url: requestUrl, json: data }, options), (err, response, data) => {
                if (err)
                    reject(err);
                resolve({
                    response,
                    data
                });
            });
        });
    }
};
HttpClient = HttpClient_1 = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [])
], HttpClient);
exports.HttpClient = HttpClient;
//# sourceMappingURL=client.js.map