"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorators_1 = require("./decorators");
const superstruct_1 = require("superstruct");
const errors_1 = require("./errors");
const enums_1 = require("./enums");
const enums_2 = require("./lib/enums");
const apiEndpointsStructure = superstruct_1.struct({
    path: 'string',
    middlewares: superstruct_1.struct.optional(['string']),
    method: superstruct_1.struct.enum(Object.values(enums_1.HTTP_METHODS)),
    controller: 'string',
    routeCache: 'number?',
    cacheControl: 'string?'
});
const spdyStructure = superstruct_1.struct({
    key: superstruct_1.struct.optional(superstruct_1.struct.union(['string', 'buffer'])),
    cert: superstruct_1.struct.optional(superstruct_1.struct.union(['string', 'buffer'])),
    passphrase: 'string',
    protocols: [superstruct_1.struct.enum(Object.values(enums_1.TRANSFER_PROTOCOLS))],
});
const customHeaderStructure = superstruct_1.struct({
    key: 'string',
    value: superstruct_1.struct.union(['string', 'number']),
    isEnv: 'boolean?'
});
const apiVersionStructure = superstruct_1.struct({
    handler: 'string?',
    endpoints: [apiEndpointsStructure]
});
const apiStructure = superstruct_1.struct({
    name: 'string',
    testCookie: 'string',
    liveVersion: 'string',
    versions: superstruct_1.struct.dict(['string', apiVersionStructure])
});
const gatewayRenderStructure = superstruct_1.struct({
    url: superstruct_1.struct.union(['string', ['string']]),
    static: 'boolean?',
    selfReplace: 'boolean?',
    placeholder: 'boolean?',
    error: 'boolean?',
    timeout: 'number?',
    middlewares: superstruct_1.struct.optional(['string']),
    routeCache: 'number?'
});
const gatewayFragmentAssetsStructure = superstruct_1.struct({
    name: 'string',
    fileName: 'string',
    link: 'string?',
    loadMethod: superstruct_1.struct.enum(Object.values(enums_2.RESOURCE_LOADING_TYPE)),
    type: superstruct_1.struct.enum(Object.values(enums_2.RESOURCE_TYPE)),
    dependent: superstruct_1.struct.optional(['string'])
});
const gatewayFragmentDependenctStructure = superstruct_1.struct({
    name: 'string',
    type: superstruct_1.struct.enum(Object.values(enums_2.RESOURCE_TYPE)),
    link: 'string?',
    preview: 'string?',
    injectType: superstruct_1.struct.optional(superstruct_1.struct.enum(Object.values(enums_1.RESOURCE_INJECT_TYPE)))
});
const gatewayFragmentVersionStructure = superstruct_1.struct({
    assets: [gatewayFragmentAssetsStructure],
    dependencies: [gatewayFragmentDependenctStructure],
    handler: 'string?'
});
const gatewayFragmentStructure = superstruct_1.struct({
    name: 'string',
    testCookie: 'string',
    prg: superstruct_1.struct.optional('boolean'),
    render: gatewayRenderStructure,
    warden: superstruct_1.struct.optional('object'),
    version: 'string',
    versionMatcher: 'string?',
    versions: superstruct_1.struct.dict(['string', gatewayFragmentVersionStructure])
});
const gatewayStructure = superstruct_1.struct({
    name: 'string',
    url: 'string',
    fragments: [gatewayFragmentStructure],
    api: [apiStructure],
    port: 'number',
    ipv4: 'boolean?',
    isMobile: 'boolean?',
    authToken: 'string?',
    fragmentsFolder: 'string',
    corsDomains: superstruct_1.struct.optional(['string']),
    corsMaxAge: 'number?',
    spdy: superstruct_1.struct.optional(spdyStructure),
    customHeaders: superstruct_1.struct.optional([customHeaderStructure])
});
const storefrontPageStructure = superstruct_1.struct({
    page: 'string?',
    html: 'string',
    url: superstruct_1.struct.union(['string', ['string']]),
    condition: 'function?'
});
const storefrontGatewaysStructure = superstruct_1.struct({
    name: 'string',
    url: 'string',
    assetUrl: 'string?'
});
const storefrontDependencyStructure = superstruct_1.struct({
    content: 'string?',
    link: 'string?'
});
const storefrontStructure = superstruct_1.struct({
    gateways: [storefrontGatewaysStructure],
    port: 'number',
    authToken: 'string?',
    satisfyUpdateCount: 'number?',
    ipv4: 'boolean?',
    pages: [storefrontPageStructure],
    pollInterval: 'number?',
    dependencies: [storefrontDependencyStructure],
    spdy: superstruct_1.struct.optional(spdyStructure),
    customHeaders: superstruct_1.struct.optional([customHeaderStructure])
});
let Configurator = class Configurator {
    constructor() {
        this.dependencies = {
            [enums_1.INJECTABLE.CUSTOM]: {},
            [enums_1.INJECTABLE.HANDLER]: {},
            [enums_1.INJECTABLE.MIDDLEWARE]: {},
        };
    }
    register(name, type, dependency) {
        this.dependencies[type][name] = dependency;
    }
    get(name, type) {
        return this.dependencies[type][name];
    }
    /**
     * Config json
     * @param {IStorefrontConfig | IGatewayBFFConfiguration} configuration
     */
    config(configuration) {
        this.validate(configuration);
        this.injectDependencies(configuration);
        this.configuration = Object.assign({}, configuration);
    }
    validate(configuration) {
        throw new errors_1.PuzzleError(errors_1.ERROR_CODES.CONFIGURATION_BASE_VALIDATION_ERROR);
    }
    injectDependencies(configuration) {
        throw new errors_1.PuzzleError(errors_1.ERROR_CODES.CONFIGURATION_BASE_VALIDATION_ERROR);
    }
};
Configurator = __decorate([
    decorators_1.sealed
], Configurator);
exports.Configurator = Configurator;
let StorefrontConfigurator = class StorefrontConfigurator extends Configurator {
    validate(configuration) {
        storefrontStructure(configuration);
    }
    injectDependencies(configuration) {
    }
};
StorefrontConfigurator = __decorate([
    decorators_1.sealed
], StorefrontConfigurator);
exports.StorefrontConfigurator = StorefrontConfigurator;
let GatewayConfigurator = class GatewayConfigurator extends Configurator {
    validate(configuration) {
        gatewayStructure(configuration);
    }
    injectDependencies(configuration) {
        this.injectCustomDependencies(configuration);
        this.injectApiHandlers(configuration);
        this.prepareFragmentConfiguration(configuration);
    }
    injectApiHandlers(configuration) {
        configuration.api.forEach(api => {
            Object.values(api.versions).forEach((version) => {
                if (version.handler) {
                    version.handler = this.dependencies[enums_1.INJECTABLE.HANDLER][version.handler];
                }
                version.endpoints.forEach((endpoint) => {
                    endpoint.middlewares = (endpoint.middlewares || []).map((middleware) => this.dependencies[enums_1.INJECTABLE.MIDDLEWARE][middleware]);
                });
            });
        });
    }
    prepareFragmentConfiguration(configuration) {
        configuration.fragments.forEach(fragment => {
            Object.values(fragment.versions).forEach((version) => {
                if (version.handler) {
                    version.handler = this.dependencies[enums_1.INJECTABLE.HANDLER][version.handler];
                }
            });
            fragment.render.middlewares = (fragment.render.middlewares || [])
                .map((middleware) => this.dependencies[enums_1.INJECTABLE.MIDDLEWARE][middleware]);
        });
    }
    injectCustomDependencies(configuration) {
        Object.keys(configuration).forEach((key) => {
            if (typeof configuration[key] !== 'object') {
                const targetDependency = this.get(configuration[key], enums_1.INJECTABLE.CUSTOM);
                if (targetDependency) {
                    configuration[key] = targetDependency;
                }
            }
            else {
                this.injectCustomDependencies(configuration[key]);
            }
        });
    }
};
GatewayConfigurator = __decorate([
    decorators_1.sealed
], GatewayConfigurator);
exports.GatewayConfigurator = GatewayConfigurator;
//# sourceMappingURL=configurator.js.map