"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const perf_hooks_1 = require("perf_hooks");
const errors_1 = require("./errors");
exports.sealed = (constructor) => {
    Object.seal(constructor);
    Object.seal(constructor.prototype);
};
exports.callableOnce = (target, name, descriptor) => {
    const newDescriptor = Object.assign({}, descriptor);
    newDescriptor.value = function () {
        if (!this[`___callableOnce__${name}___`]) {
            descriptor.value.apply(this, arguments);
            this[`___callableOnce__${name}___`] = true;
        }
        else {
            throw new errors_1.PuzzleError(errors_1.ERROR_CODES.CALLABLE_ONCE_CALLED_MORE_THAN_ONE_TIME, target.constructor.name, name);
        }
    };
    return newDescriptor;
};
exports.benchmark = (enabled, logger) => {
    return (target, propertyKey, descriptor) => {
        if (!enabled)
            return descriptor;
        console.log(`Attaching benchmark to ${propertyKey}`);
        const newDescriptor = Object.assign({}, descriptor);
        newDescriptor.value = function () {
            const old_time = perf_hooks_1.performance.now();
            const m0 = process.memoryUsage();
            const c0 = process.cpuUsage();
            const returnValue = descriptor.value.apply(this, arguments);
            const new_time = perf_hooks_1.performance.now();
            const m1 = process.memoryUsage();
            const diffCPU = process.cpuUsage(c0);
            logger({
                benchmarking: `${this.constructor.name} - ${propertyKey}`,
                ram: `${(m1.rss - m0.rss) / 1048576} mb`,
                heapTotal: `${(m1.heapTotal - m0.heapTotal) / 1048576} mb`,
                heapUsed: `${(m1.heapUsed - m0.heapUsed) / 1048576} mb`,
                cpu: `${(diffCPU.user + diffCPU.system) / 1000000} s`,
                spendTime: `${(new_time - old_time)} ms`
            });
            return returnValue;
        };
        return newDescriptor;
    };
};
//# sourceMappingURL=decorators.js.map