"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ERROR_CODES;
(function (ERROR_CODES) {
    ERROR_CODES[ERROR_CODES["TEMPLATE_NOT_FOUND"] = 0] = "TEMPLATE_NOT_FOUND";
    ERROR_CODES[ERROR_CODES["MULTIPLE_PRIMARY_FRAGMENTS"] = 1] = "MULTIPLE_PRIMARY_FRAGMENTS";
    ERROR_CODES[ERROR_CODES["CONFIGURATION_BASE_VALIDATION_ERROR"] = 2] = "CONFIGURATION_BASE_VALIDATION_ERROR";
    ERROR_CODES[ERROR_CODES["CALLABLE_ONCE_CALLED_MORE_THAN_ONE_TIME"] = 3] = "CALLABLE_ONCE_CALLED_MORE_THAN_ONE_TIME";
    ERROR_CODES[ERROR_CODES["FAILED_TO_COMPILE_TEMPLATE"] = 4] = "FAILED_TO_COMPILE_TEMPLATE";
    ERROR_CODES[ERROR_CODES["FAILED_TO_GET_FRAGMENT_CONTENT"] = 5] = "FAILED_TO_GET_FRAGMENT_CONTENT";
})(ERROR_CODES = exports.ERROR_CODES || (exports.ERROR_CODES = {}));
const ERROR_MESSAGES = Object.freeze({
    [ERROR_CODES.TEMPLATE_NOT_FOUND]: () => 'Template not found in html file',
    [ERROR_CODES.MULTIPLE_PRIMARY_FRAGMENTS]: () => 'Multiple primary fragments are not allowed',
    [ERROR_CODES.CONFIGURATION_BASE_VALIDATION_ERROR]: () => 'Use StorefrontConfiguration or GatewayConfiguration',
    [ERROR_CODES.CALLABLE_ONCE_CALLED_MORE_THAN_ONE_TIME]: (target, method) => `You can't call method ${method} of ${target} more than one once`,
    [ERROR_CODES.FAILED_TO_COMPILE_TEMPLATE]: (template) => `Failed to compile template: ${template}`,
    [ERROR_CODES.FAILED_TO_GET_FRAGMENT_CONTENT]: (fragmentName, url) => `Failed to get fragment contents for fragment ${fragmentName}, link: ${url}`
});
class PuzzleError extends Error {
    constructor(ERROR_CODE, ...args) {
        super(ERROR_MESSAGES[ERROR_CODE].apply(null, args));
        Object.setPrototypeOf(this, PuzzleError.prototype);
    }
}
exports.PuzzleError = PuzzleError;
//# sourceMappingURL=errors.js.map