/// <reference types="node" />
import { HandlerDataResponse, ICookieMap, IExposeFragment, IFragment, IFragmentBFF, IFragmentContentResponse } from "./types";
import { Request } from 'express';
import express from "express";
import { CookieVersionMatcher } from "./cookie-version-matcher";
export declare class Fragment {
    name: string;
    constructor(config: IFragment);
}
export declare class FragmentBFF extends Fragment {
    config: IFragmentBFF;
    versionMatcher?: CookieVersionMatcher;
    private handler;
    constructor(config: IFragmentBFF);
    /**
     * Renders fragment: data -> content
     * @param {object} req
     * @param {string} version
     * @returns {Promise<HandlerDataResponse>}
     */
    render(req: express.Request, version: string): Promise<HandlerDataResponse>;
    /**
     * Renders placeholder
     * @param {object} req
     * @param {string} version
     * @returns {string}
     */
    placeholder(req: object, version?: string): string;
    /**
     * Renders error
     * @param {object} req
     * @param {string} version
     * @returns {string}
     */
    errorPage(req: object, version?: string): {
        main: string;
    };
    /**
     * Purifies req.path, req.query from Puzzle elements.
     * @param req
     * @returns {*}
     */
    private clearRequest;
    /**
     * Check module type
     */
    private checkModuleType;
    /**
     * Resolve handlers based on configuration
     */
    private prepareHandlers;
}
export declare class FragmentStorefront extends Fragment {
    config: IExposeFragment | undefined;
    primary: boolean;
    shouldWait: boolean;
    from: string;
    gatewayPath: string;
    fragmentUrl: string | undefined;
    assetUrl: string | undefined;
    private versionMatcher?;
    private cachedErrorPage;
    private gatewayName;
    constructor(name: string, from: string);
    /**
     * Updates fragment configuration
     * @param {IExposeFragment} config
     * @param {string} gatewayUrl
     * @param gatewayName
     * @param {string | undefined} assetUrl
     */
    update(config: IExposeFragment, gatewayUrl: string, gatewayName: string, assetUrl?: string | undefined): void;
    detectVersion(cookie: ICookieMap, preCompile?: boolean): string;
    /**
     * Returns fragment placeholder as promise, fetches from gateway
     * @returns {Promise<string>}
     */
    getPlaceholder(): Promise<string>;
    /**
     * Returns fragment error as promise, fetches from gateway
     * @returns { Promise<string> }
     */
    getErrorPage(): Promise<string>;
    /**
     * Fetches fragment content as promise, fetches from gateway
     * Returns {
     *  html: {
     *    Partials
     *  },
     *  status: gateway status response code
     * }
     * @param attribs
     * @param req
     * @returns {Promise<IFragmentContentResponse>}
     */
    getContent(attribs?: any, req?: Request): Promise<IFragmentContentResponse>;
    /**
     * Returns asset content
     * @param {string} name
     * @param targetVersion
     * @returns {Promise<string>}
     */
    getAsset(name: string, targetVersion: string): Promise<string | Buffer | null>;
    /**
     * Returns asset path
     * @param {string} name
     * @returns {string}
     */
    getAssetPath(name: string): string | null;
}
