import { FRAGMENT_RENDER_MODES } from "./enums";
import { ICookieMap, IExposeConfig, IGatewayBFFConfiguration } from "./types";
import express from "express";
import { Server } from "./server";
import { GatewayConfigurator } from "./configurator";
export declare class GatewayBFF {
    readonly url: string;
    readonly name: string;
    exposedConfig: IExposeConfig;
    server: Server;
    private config;
    private fragments;
    private apis;
    ready: boolean;
    /**
     * Gateway constructor
     * @param {IGatewayBFFConfiguration} gatewayConfig
     * @param {Server} _server
     */
    constructor(gatewayConfig: IGatewayBFFConfiguration | GatewayConfigurator, _server?: Server);
    /**
     * Starts gateway
     */
    init(cb?: Function): void;
    /**
     * Adds api routes
     * @param {Function} cb
     */
    private addApiRoutes;
    /**
     * Creates expose config
     * @returns {IExposeConfig}
     */
    private createExposeConfig;
    /**
     * Renders a fragment with desired version and renderMode
     * @param req
     * @param {string} fragmentName
     * @param {FRAGMENT_RENDER_MODES} renderMode
     * @param {string} partial
     * @param res
     * @param cookie
     * @param {string} forcedVersion
     * @returns {Promise<IFragmentResponse>}
     */
    renderFragment(req: express.Request, fragmentName: string, renderMode: FRAGMENT_RENDER_MODES | undefined, partial: string, res: express.Response, cookie: ICookieMap, forcedVersion?: string): Promise<void>;
    /**
     * Wraps with html template for preview mode
     * @param {string} htmlContent
     * @param {FragmentBFF} fragment
     * @param version
     * @param model
     * @returns {string}
     */
    private wrapFragmentContent;
    private detectVersion;
    /**
     * Adds fragment routes
     * @param {Function} cb
     */
    private addFragmentRoutes;
    private getFragmentMiddlewares;
    /**
     * Adds placeholder routes
     * @param {Function} cb
     */
    private addPlaceholderRoutes;
    /**
     *  Adds error routes
     *  @param { Function } cb
     */
    private addErrorPageRoutes;
    /**
     * Adds static routes
     * @param {Function} cb
     */
    private addStaticRoutes;
    /**
     * Adds healthcheck route
     * @param {Function} cb
     */
    private addHealthCheckRoutes;
    private addCorsPlugin;
    /**
     * Adds expose configuration route
     * @param {Function} cb
     */
    private addConfigurationRoute;
    /**
     * Adds custom headers
     * @param {Function} cb
     */
    private addCustomHeaders;
    /**
     * Starts gateway and configures dependencies
     */
    private bootstrap;
}
