"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fragment_1 = require("./fragment");
const api_1 = require("./api");
const enums_1 = require("./enums");
const config_1 = require("./config");
const md5_1 = __importDefault(require("md5"));
const async_1 = __importDefault(require("async"));
const path_1 = __importDefault(require("path"));
const server_1 = require("./server");
const base_1 = require("./base");
const cheerio_1 = __importDefault(require("cheerio"));
const decorators_1 = require("./decorators");
const template_1 = require("./template");
const cors_1 = __importDefault(require("cors"));
const route_cache_1 = __importDefault(require("route-cache"));
const enums_2 = require("./lib/enums");
const fs_1 = __importDefault(require("fs"));
const resource_injector_1 = __importDefault(require("./resource-injector"));
const logger = base_1.container.get(base_1.TYPES.Logger);
let GatewayBFF = class GatewayBFF {
    /**
     * Gateway constructor
     * @param {IGatewayBFFConfiguration} gatewayConfig
     * @param {Server} _server
     */
    constructor(gatewayConfig, _server) {
        this.fragments = {};
        this.apis = {};
        this.ready = false;
        this.server = _server || base_1.container.get(base_1.TYPES.Server);
        this.config = gatewayConfig.hasOwnProperty('configuration') ? gatewayConfig.configuration : gatewayConfig;
        this.bootstrap();
    }
    get url() {
        return this.config.url;
    }
    get name() {
        return this.config.name;
    }
    /**
     * Starts gateway
     */
    init(cb) {
        async_1.default.series([
            this.addCorsPlugin.bind(this),
            this.addCustomHeaders.bind(this),
            this.addPlaceholderRoutes.bind(this),
            this.addErrorPageRoutes.bind(this),
            this.addApiRoutes.bind(this),
            this.addStaticRoutes.bind(this),
            this.addHealthCheckRoutes.bind(this),
            this.addFragmentRoutes.bind(this),
            this.addConfigurationRoute.bind(this)
        ], err => {
            if (!err) {
                logger.info(`Gateway is listening on port ${this.config.port}`);
                this.server.listen(this.config.port, cb, this.config.ipv4);
            }
            else {
                throw err;
            }
        });
    }
    /**
     * Adds api routes
     * @param {Function} cb
     */
    addApiRoutes(cb) {
        this.config.api.forEach(apiConfig => {
            this.apis[apiConfig.name] = new api_1.Api(apiConfig);
            this.apis[apiConfig.name].registerEndpoints(this.server);
        });
        cb();
    }
    /**
     * Creates expose config
     * @returns {IExposeConfig}
     */
    createExposeConfig() {
        return {
            fragments: this.config.fragments.reduce((fragmentList, fragment) => {
                fragmentList[fragment.name] = {
                    version: fragment.version,
                    render: fragment.render,
                    assets: fragment.versions[fragment.version].assets,
                    dependencies: fragment.versions[fragment.version].dependencies,
                    testCookie: fragment.testCookie,
                    prg: !!fragment.prg,
                    passiveVersions: Object.keys(fragment.versions).filter(v => v !== fragment.version).reduce((versionInfo, version) => (Object.assign({}, versionInfo, { [version]: {
                            assets: fragment.versions[version].assets,
                            dependencies: fragment.versions[version].dependencies
                        } })), {})
                };
                if (fragment.warden) {
                    fragmentList[fragment.name].warden = fragment.warden;
                }
                if (fragment.versionMatcher) {
                    fragmentList[fragment.name].versionMatcher = fragment.versionMatcher.toString();
                }
                this.fragments[fragment.name] = new fragment_1.FragmentBFF(fragment);
                return fragmentList;
            }, {}),
            hash: '',
        };
    }
    /**
     * Renders a fragment with desired version and renderMode
     * @param req
     * @param {string} fragmentName
     * @param {FRAGMENT_RENDER_MODES} renderMode
     * @param {string} partial
     * @param res
     * @param cookie
     * @param {string} forcedVersion
     * @returns {Promise<IFragmentResponse>}
     */
    renderFragment(req, fragmentName, renderMode = enums_1.FRAGMENT_RENDER_MODES.PREVIEW, partial, res, cookie, forcedVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            const fragment = this.fragments[fragmentName];
            if (fragment) {
                const version = this.detectVersion(fragment, cookie, forcedVersion);
                const fragmentContent = yield fragment.render(req, version);
                const gatewayContent = {
                    content: fragmentContent,
                    $status: +(fragmentContent.$status || enums_1.HTTP_STATUS_CODE.OK),
                    $headers: fragmentContent.$headers || {},
                    $cookies: fragmentContent.$cookies || {},
                    $model: fragmentContent.$model
                };
                Object.keys(gatewayContent.$headers).forEach(key => {
                    res.set(key, gatewayContent.$headers[key]);
                });
                Object.keys(gatewayContent.$cookies).forEach(key => {
                    res.cookie(key, gatewayContent.$cookies[key].value, gatewayContent.$cookies[key].options);
                });
                if (renderMode === enums_1.FRAGMENT_RENDER_MODES.STREAM) {
                    res.status(enums_1.HTTP_STATUS_CODE.OK);
                    res.json(gatewayContent.content);
                }
                else {
                    if (gatewayContent.$status === enums_1.HTTP_STATUS_CODE.MOVED_PERMANENTLY && gatewayContent.$headers && gatewayContent.$headers['location']) {
                        res.status(gatewayContent.$status);
                        res.end();
                    }
                    else {
                        if (gatewayContent.content[partial]) {
                            res.status(enums_1.HTTP_STATUS_CODE.OK);
                            res.send(this.wrapFragmentContent(gatewayContent.content[partial].toString(), fragment, version, gatewayContent.$model));
                        }
                        else {
                            res.status(enums_1.HTTP_STATUS_CODE.INTERNAL_SERVER_ERROR);
                            res.send(`Partial ${partial} doesn't exist in fragment response`);
                        }
                    }
                }
            }
            else {
                throw new Error(`Failed to find fragment: ${fragmentName}`);
            }
        });
    }
    /**
     * Wraps with html template for preview mode
     * @param {string} htmlContent
     * @param {FragmentBFF} fragment
     * @param version
     * @param model
     * @returns {string}
     */
    wrapFragmentContent(htmlContent, fragment, version, model) {
        const dom = cheerio_1.default.load(`<html><head><title>${this.config.name} - ${fragment.name}</title>${this.config.isMobile ? '<meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no" />' : ''}${template_1.Template.fragmentModelScript(fragment, model, false)}</head><body><div id="${fragment.name}">${htmlContent}</div></body></html>`);
        const fragmentVersion = fragment.config.versions[version];
        dom('head').prepend(resource_injector_1.default.wrapJsAsset({
            content: fs_1.default.readFileSync(path_1.default.join(__dirname, `/lib/puzzle.min.js`)).toString(),
            injectType: enums_1.RESOURCE_INJECT_TYPE.INLINE,
            name: 'puzzle-lib',
            link: '',
            executeType: enums_1.RESOURCE_JS_EXECUTE_TYPE.SYNC
        }));
        fragmentVersion.assets.forEach(asset => {
            if (asset.type === enums_2.RESOURCE_TYPE.JS) {
                dom('body').append(`<script puzzle-asset="${asset.name}" src="/${fragment.name}/static/${asset.fileName}" type="text/javascript"${enums_1.RESOURCE_JS_EXECUTE_TYPE.SYNC}></script>`);
            }
            else if (asset.type === enums_2.RESOURCE_TYPE.CSS) {
                dom('head').append(`<link puzzle-asset="${asset.name}" rel="stylesheet" href="/${fragment.name}/static/${asset.fileName}" />`);
            }
        });
        fragmentVersion.dependencies.forEach(dependency => {
            if (dependency.type === enums_2.RESOURCE_TYPE.JS) {
                dom('head').append(`<script puzzle-asset="${dependency.name}" src="${dependency.preview}" type="text/javascript"></script>`);
            }
            else if (dependency.type === enums_2.RESOURCE_TYPE.CSS) {
                dom('head').append(`<link puzzle-asset="${dependency.name}" rel="stylesheet" href="${dependency.preview}" />`);
            }
        });
        return dom.html();
    }
    detectVersion(fragment, cookie, forcedVersion) {
        const cookieKey = fragment.config.testCookie;
        let cookieVersion;
        if (forcedVersion && fragment.config.versions[forcedVersion]) {
            cookieVersion = forcedVersion;
        }
        else if (cookie[cookieKey] && fragment.config.versions[cookie[cookieKey]]) {
            cookieVersion = cookie[cookieKey];
        }
        if (cookieVersion)
            return cookieVersion;
        const matcherVersion = fragment.versionMatcher ? fragment.versionMatcher.match(cookie) : null;
        if (matcherVersion && fragment.config.versions[matcherVersion])
            return matcherVersion;
        return fragment.config.version;
    }
    /**
     * Adds fragment routes
     * @param {Function} cb
     */
    addFragmentRoutes(cb) {
        this.config.fragments.forEach(fragmentConfig => {
            this.server.addRoute(Array.isArray(fragmentConfig.render.url) ? fragmentConfig.render.url.map(url => `/${fragmentConfig.name}${url}`) : `/${fragmentConfig.name}${fragmentConfig.render.url}`, enums_1.HTTP_METHODS.GET, (req, res) => __awaiter(this, void 0, void 0, function* () {
                const partial = req.query[config_1.PREVIEW_PARTIAL_QUERY_NAME] || enums_1.DEFAULT_MAIN_PARTIAL;
                const renderMode = req.query[config_1.RENDER_MODE_QUERY_NAME] === enums_1.FRAGMENT_RENDER_MODES.STREAM ? enums_1.FRAGMENT_RENDER_MODES.STREAM : enums_1.FRAGMENT_RENDER_MODES.PREVIEW;
                req.headers['originalurl'] = req.headers['originalurl'] || req.url.replace(`/${fragmentConfig.name}`, "");
                req.headers['originalpath'] = req.headers['originalpath'] || req.path.replace(`/${fragmentConfig.name}`, "");
                this.renderFragment(req, fragmentConfig.name, renderMode, partial, res, req.cookies, req.query[config_1.VERSION_QUERY_NAME]);
            }), this.getFragmentMiddlewares(fragmentConfig));
        });
        cb();
    }
    getFragmentMiddlewares(fragmentConfig) {
        const fragmentMiddlewares = fragmentConfig.render.middlewares || [];
        if (fragmentConfig.render.routeCache) {
            fragmentMiddlewares.unshift(route_cache_1.default.cacheSeconds(+fragmentConfig.render.routeCache));
        }
        return fragmentMiddlewares;
    }
    /**
     * Adds placeholder routes
     * @param {Function} cb
     */
    addPlaceholderRoutes(cb) {
        this.config.fragments.forEach(fragment => {
            this.server.addRoute(`/${fragment.name}/placeholder`, enums_1.HTTP_METHODS.GET, (req, res) => __awaiter(this, void 0, void 0, function* () {
                if (req.query.delay && +req.query.delay) {
                    res.set('content-type', 'text/html');
                    const dom = cheerio_1.default.load(`<html><head><title>${this.config.name} - ${fragment.name}</title>${this.config.isMobile ? '<meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no" />' : ''}</head><body><div id="${fragment.name}">${this.fragments[fragment.name].placeholder(req, req.cookies[fragment.testCookie])}</div></body></html>`);
                    res.write(dom.html());
                    const gatewayContent = yield this.fragments[fragment.name].render(req, req.cookies[fragment.testCookie]);
                    res.write(`${enums_1.CONTENT_REPLACE_SCRIPT}<div style="display: none;" id="${fragment.name}-replace">${gatewayContent[enums_1.DEFAULT_MAIN_PARTIAL]}</div>`);
                    setTimeout(() => {
                        res.write(`<script>$p('#${fragment.name}', '#${fragment.name}-replace')</script>`);
                        res.end();
                    }, +req.query.delay);
                }
                else {
                    res.send(this.fragments[fragment.name].placeholder(req, req.cookies[fragment.testCookie]));
                }
            }));
        });
        cb();
    }
    /**
     *  Adds error routes
     *  @param { Function } cb
     */
    addErrorPageRoutes(cb) {
        this.config.fragments.forEach((fragment) => {
            this.server.addRoute(`/${fragment.name}/error`, enums_1.HTTP_METHODS.GET, (req, res) => {
                res.send(this.fragments[fragment.name].errorPage(req, req.cookies[fragment.testCookie]));
            });
        });
        cb();
    }
    /**
     * Adds static routes
     * @param {Function} cb
     */
    addStaticRoutes(cb) {
        Object.values(this.fragments).forEach(fragment => {
            const config = fragment.config;
            this.server.addRoute(`/${fragment.name}/static/:staticName`, enums_1.HTTP_METHODS.GET, (req, res, next) => {
                const targetVersion = req.query['__version'] || this.detectVersion(fragment, req.cookies);
                req.url = path_1.default.join('/', fragment.name, req.cookies[config.testCookie] || targetVersion, '/static/', req.params.staticName);
                next();
            });
            Object.keys(config.versions).forEach(version => {
                const staticPath = path_1.default.join(this.config.fragmentsFolder, config.name, version, '/assets');
                this.server.setStatic(`/${config.name}/${version}/static/`, staticPath);
            });
        });
        cb();
    }
    /**
     * Adds healthcheck route
     * @param {Function} cb
     */
    addHealthCheckRoutes(cb) {
        this.server.addRoute(enums_1.HEALTHCHECK_PATHS, enums_1.HTTP_METHODS.GET, (req, res) => {
            res.status(enums_1.HTTP_STATUS_CODE.OK).end();
        });
        cb();
    }
    addCorsPlugin(cb) {
        this.server.addUse(null, cors_1.default({
            origin: this.config.corsDomains || ['*'],
            credentials: true,
            maxAge: this.config.corsMaxAge || undefined
        }));
        cb();
    }
    /**
     * Adds expose configuration route
     * @param {Function} cb
     */
    addConfigurationRoute(cb) {
        this.server.addRoute('/', enums_1.HTTP_METHODS.GET, (req, res) => {
            if (req.query.fragment) {
                const fragment = this.exposedConfig.fragments[req.query.fragment];
                if (fragment) {
                    return res.status(enums_1.HTTP_STATUS_CODE.OK).json({
                        assets: fragment.assets,
                        dependencies: fragment.dependencies,
                        version: fragment.version,
                        passiveVersions: fragment.passiveVersions
                    });
                }
                return res.status(enums_1.HTTP_STATUS_CODE.NOT_FOUND).end();
            }
            if (!this.config.authToken || req.header('x-authorization') === this.config.authToken) {
                res.status(enums_1.HTTP_STATUS_CODE.OK).json(this.exposedConfig);
            }
            else {
                res.status(enums_1.HTTP_STATUS_CODE.UNAUTHORIZED).end();
            }
        });
        cb();
    }
    /**
     * Adds custom headers
     * @param {Function} cb
     */
    addCustomHeaders(cb) {
        this.server.addCustomHeaders(this.config.customHeaders);
        cb();
    }
    /**
     * Starts gateway and configures dependencies
     */
    bootstrap() {
        this.server.useProtocolOptions(this.config.spdy);
        this.exposedConfig = this.createExposeConfig();
        this.exposedConfig.hash = md5_1.default(JSON.stringify(this.exposedConfig));
    }
};
__decorate([
    decorators_1.callableOnce,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Function]),
    __metadata("design:returntype", void 0)
], GatewayBFF.prototype, "init", null);
GatewayBFF = __decorate([
    decorators_1.sealed,
    __metadata("design:paramtypes", [Object, server_1.Server])
], GatewayBFF);
exports.GatewayBFF = GatewayBFF;
//# sourceMappingURL=gatewayBFF.js.map