"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const winston = require('winston');
const WinstonGraylog2 = require('winston-graylog2');
const logLevels = winston.config.npm.levels;
/**
 * Logs Levels
 */
const alignedWithColorsAndTime = winston.format.combine(winston.format.colorize(), winston.format.timestamp(), winston.format.align(), winston.format.printf((info) => {
    const { timestamp, level, message } = info, args = __rest(info, ["timestamp", "level", "message"]);
    const ts = timestamp.slice(0, 19).replace('T', ' ');
    return `${ts} [${level}]: ${message} ${Object.keys(args).length ? JSON.stringify(args, null, 2) : ''}`;
}));
/**
 * Creates winston transports
 * @returns {Array}
 */
const createTransports = () => {
    const transports = [];
    if (process.env.ENABLE_CONSOLE_ERROR || process.env.NODE_ENV != 'production' || process.env.ENVIRONMENT != "production") {
        transports.push(new winston.transports.Console({
            level: process.env.logLevel || 'warn',
            handleExceptions: false,
            json: true,
            colorize: true,
            format: alignedWithColorsAndTime
        }));
    }
    if (process.env.GRAYLOG_HOST && process.env.GRAYLOG_PORT && process.env.GRAYLOG_HOSTNAME && process.env.GRAYLOG_FACILITY) {
        transports.push(new WinstonGraylog2({
            name: 'Graylog',
            level: process.env.logLevel || 'warn',
            silent: false,
            handleExceptions: false,
            graylog: {
                servers: [{ host: process.env.GRAYLOG_HOST, port: +process.env.GRAYLOG_PORT }],
                hostname: process.env.GRAYLOG_HOSTNAME,
                facility: process.env.GRAYLOG_FACILITY,
                bufferSize: 1400,
            },
            staticMeta: {}
        }));
    }
    return transports;
};
Object.defineProperty(Error.prototype, 'toJSON', {
    value() {
        const alt = {};
        const propNames = Object.getOwnPropertyNames(this);
        for (let i = 0, len = propNames.length; i < len; i++) {
            const key = propNames[i];
            alt[key] = this[key];
        }
        return alt;
    },
    configurable: true,
    writable: true
});
/**
 * Create global logger
 */
const _logger = winston.createLogger({
    transports: createTransports()
});
exports.logger = {};
Object.keys(logLevels).forEach(level => {
    exports.logger[level] = function () {
        let log = ``;
        for (let i = 0, len = arguments.length; i < len; i++) {
            log += `${JSON.stringify(arguments[i], null, 4)}\r\n`;
        }
        _logger[level](log);
    };
});
/**
 * Register to error events
 */
if (process.env.NODE_ENV === 'production') {
    process
        .on('unhandledRejection', (reason, p) => {
        exports.logger.error('UN_HANDLED_REJECTION', reason, p);
    })
        .on('uncaughtException', err => {
        exports.logger.error('UN_CAUGHT_EXCEPTION', err);
    });
}
let Logger = class Logger {
    constructor() {
        this.logger = exports.logger;
        Object.keys(logLevels).forEach(level => {
            this[level] = exports.logger[level];
        });
    }
    write(message) {
        let level = 'info';
        if (message.match(/RES: [4]/)) {
            level = 'warn';
        }
        else if (message.match(/RES: [5]/)) {
            level = 'error';
        }
        exports.logger[level](message);
    }
};
Logger = __decorate([
    inversify_1.injectable(),
    __metadata("design:paramtypes", [])
], Logger);
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map