import { Template } from "./template";
import { ICookieObject, IGatewayMap, IPageDependentGateways, IResponseHandlers } from "./types";
import express from "express";
export declare class Page {
    name: string;
    ready: boolean;
    gatewayDependencies: IPageDependentGateways;
    responseHandlers: IResponseHandlers;
    template: Template;
    private waitingCompilers;
    private rawHtml;
    private prgEnabled;
    constructor(html: string, gatewayMap: IGatewayMap, name: string);
    /**
     * Request handler, compiles template if not exists
     * @param {{cookies: ICookieObject}} req
     * @param {object} res
     * @returns {Promise<void>}
     */
    handle(req: {
        cookies: ICookieObject;
        query: {
            [name: string]: string;
        };
    }, res: object): Promise<void>;
    post(req: express.Request, res: express.Response, next: express.NextFunction): void;
    reCompile(): Promise<void>;
    /**
     * Prepares dependencies and subscribes to events.
     * @param {IGatewayMap} gatewayMap
     */
    private preparePageDependencies;
    /**
     * Checks for page dependencies are ready
     */
    private checkPageReady;
    /**
     * Based on test cookies returns handler key
     * @returns {string}
     * @param query
     * @param cookies
     */
    private getHandlerVersion;
    private updatePrgStatus;
    /**
     * Called on GATEWAY_UPDATED
     * @param {GatewayStorefrontInstance} gateway
     */
    private gatewayUpdated;
    /**
     * Updates fragment config based on gateway configuration
     * @param {GatewayStorefrontInstance} gateway
     */
    private updateFragmentsConfig;
    /**
     * Called on GATEWAY_READY
     * @param {GatewayStorefrontInstance} gateway
     */
    private gatewayReady;
}
