import { FragmentStorefront } from "./fragment";
import { ICookieMap, IWrappingJsAsset } from "./types";
export default class ResourceInjector {
    private readonly fragments;
    private readonly pageName;
    private readonly cookies;
    private readonly fragmentFingerPrints;
    private readonly assets;
    private readonly dependencies;
    private libraryConfig;
    constructor(fragments: {
        [name: string]: FragmentStorefront;
    }, pageName: string | undefined, cookies: ICookieMap);
    /**
     * Injects prepared assets to dom
     * @param { CheerioStatic } dom
     * @returns {Promise<void>}
     */
    injectAssets(dom: CheerioStatic): void;
    /**
     * Injects library config to dom
     * @param { CheerioStatic } dom
     * @returns {Promise<void>}
     */
    injectLibraryConfig(dom: CheerioStatic): void;
    /**
     * Merges, minifies stylesheets and inject them to dom
     * @param { CheerioStatic } dom
     * @param { boolean } precompile
     * @returns {Promise<void>}
     */
    injectStyleSheets(dom: CheerioStatic, precompile: boolean): Promise<unknown>;
    /**
     * Wraps js asset based on its configuration
     * @param {IWrappingJsAsset} asset
     * @returns {string}
     */
    static wrapJsAsset(asset: IWrappingJsAsset): string;
    /**
     * Load css data(stylesheets and dependency name list) and push to first parameter
     * @param { } cssData
     * @param {FragmentStorefront} fragment
     * @param {boolean} precompile
     * @returns {Promise<>}
     */
    private loadCSSData;
    /**
     * Returns fragment config using targeted version
     * @param { FragmentStorefront } fragment
     * @param { string } targetVersion
     * @returns { }
     */
    private getFragmentConfig;
    /**
     * Prepare assets, fragmentFingerPrints, dependencies and library config
     * @returns {Promise<void>}
     */
    private prepare;
    /**
     * Prepare injectable assets from fragment and push to this.assets
     * @param { } config
     */
    private prepareAssets;
    /**
     * Prepare injectable dependencies from fragment and push to this.dependencies
     * @param { } config
     * @returns {IPageFragmentConfig}
     */
    private prepareDependencies;
    /**
     * Prepare fragment fingerprint from given fragment and push to this.fragmentFingerPrints
     * @param {FragmentStorefront} fragment
     */
    private prepareFragmentFingerPrint;
    /**
     * Inject default JS assets
     * @param { } jsAsset
     * @param { CheerioStatic } dom
     * @returns {string}
     */
    private static injectDefaultJsAsset;
}
