/// <reference types="node" />
import { Express } from "express";
import "reflect-metadata";
import * as Http from "http";
import { NextFunction, Request, Response } from "express";
import { ServeStaticOptions } from "serve-static";
import { HTTP_METHODS } from "./enums";
import { ICustomHeader, ISpdyConfiguration } from "./types";
import * as spdy from "spdy";
export declare class Server {
    app: Express;
    server: Http.Server | spdy.Server | null;
    private spdyConfiguration;
    constructor();
    /**
     * Sets spdy protocol configuration.
     * @param {ISpdyConfiguration} options
     */
    useProtocolOptions(options?: ISpdyConfiguration): void;
    /**
     * Register new middleware to route
     * @param {string | null} path
     * @param {(req: Request, res: Response, next: NextFunction) => any} handler
     */
    addUse(path: string | null, handler: (req: Request, res: Response, next: NextFunction) => any): void;
    /**
     * Registers static routes
     * @param {string | null} path
     * @param {string} source
     * @param {serveStatic.ServeStaticOptions} staticOptions
     */
    setStatic(path: string | null, source: string, staticOptions?: ServeStaticOptions): void;
    /**
     * Adds new route
     * @param {string} path
     * @param {HTTP_METHODS} method
     * @param {(req: Request, res: Response, next: NextFunction) => any} handler
     * @param {RequestHandlerParams[]} middlewares
     */
    addRoute(path: string | string[], method: HTTP_METHODS, handler: (req: Request, res: Response, next: NextFunction) => any, middlewares?: any[]): void;
    /**
     * Starts server
     * @param {number} port
     * @param {Function} cb
     * @param ipv4
     */
    listen(port: number, cb?: Function, ipv4?: boolean): void;
    /**
     * Adds custom headers
     * @param {Array<ICustomHeader>} customHeaders
     */
    addCustomHeaders(customHeaders?: ICustomHeader[]): void;
    /**
     * Clears instances, stops listening
     */
    close(): void;
    /**
     * Predefined middlewares
     * @returns {boolean}
     */
    private addMiddlewares;
}
