import { Server } from "./server";
import { IGatewayMap, IPageMap, IStorefrontConfig } from "./types";
import { StorefrontConfigurator } from "./configurator";
export declare class Storefront {
    server: Server;
    config: IStorefrontConfig;
    pages: IPageMap;
    gateways: IGatewayMap;
    private gatewaysReady;
    /**
     * Storefront Instance
     * @param {IStorefrontConfig} storefrontConfig
     * @param {Server} _server
     */
    constructor(storefrontConfig: IStorefrontConfig | StorefrontConfigurator, _server?: Server);
    /**
     * Starts storefront instance
     * @param {Function} cb
     */
    init(cb?: Function): void;
    private preLoadPages;
    private bootstrap;
    /**
     * Creates static routes for debugging scripts
     * @param {Function} cb
     * @returns {Promise<void>}
     */
    private registerDebugScripts;
    /**
     * Waits for gateways to be prepared
     * @param {Function} cb
     * @returns {Promise<void>}
     */
    private waitForGateways;
    /**
     * Creates gateway pages, pages and subscribes event to gateways to track ready status
     */
    private createStorefrontPagesAndGateways;
    /**
     * Registers provided dependencies in storefront configuration
     * @param {Function} cb
     */
    private registerDependencies;
    /**
     * Adds healthcheck route.
     * @param {Function} cb
     */
    private addHealthCheckRoute;
    /**
     * Adds custom headers
     * @param {Function} cb
     */
    private addCustomHeaders;
    /**
     * Adds page routes then connects with page instance responsible for it.
     * @param {Function} cb
     */
    private addPageRoute;
}
