"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fragment_1 = require("./fragment");
const cheerio_1 = __importDefault(require("cheerio"));
const templateCompiler_1 = require("./templateCompiler");
const config_1 = require("./config");
const enums_1 = require("./enums");
const resource_injector_1 = __importDefault(require("./resource-injector"));
const util_1 = require("./util");
const templateClass_1 = require("./templateClass");
const errors_1 = require("./errors");
const decorators_1 = require("./decorators");
const base_1 = require("./base");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const enums_2 = require("./lib/enums");
const logger = base_1.container.get(base_1.TYPES.Logger);
class Template {
    constructor(rawHtml, name) {
        this.rawHtml = rawHtml;
        this.name = name;
        this.fragments = {};
        this.pageClass = new templateClass_1.TemplateClass();
        this.load();
        this.bindPageClass();
        this.pageClass._onCreate();
    }
    /**
     * Loads html template into Cheerio instance
     */
    load() {
        const templateMatch = templateCompiler_1.TemplateCompiler.TEMPLATE_CONTENT_REGEX.exec(this.rawHtml);
        if (templateMatch) {
            this.dom = cheerio_1.default.load(Template.replaceCustomScripts(templateMatch[1], true), config_1.CHEERIO_CONFIGURATION);
        }
        else {
            throw new errors_1.PuzzleError(errors_1.ERROR_CODES.TEMPLATE_NOT_FOUND);
        }
    }
    /**
     * Returns fragment dependencies
     * @returns {IPageDependentGateways}
     */
    getDependencies() {
        let primaryName;
        return this.dom(config_1.TEMPLATE_FRAGMENT_TAG_NAME).toArray().reduce((dependencyList, fragment) => {
            if (!dependencyList.gateways[fragment.attribs.from]) {
                dependencyList.gateways[fragment.attribs.from] = {
                    gateway: null,
                    ready: false
                };
            }
            if (!dependencyList.fragments[fragment.attribs.name]) {
                this.fragments[fragment.attribs.name] = new fragment_1.FragmentStorefront(fragment.attribs.name, fragment.attribs.from);
                dependencyList.fragments[fragment.attribs.name] = {
                    gateway: fragment.attribs.from,
                    instance: this.fragments[fragment.attribs.name]
                };
            }
            if (!this.fragments[fragment.attribs.name].primary) {
                if (typeof fragment.attribs.primary !== 'undefined') {
                    if (primaryName != null && primaryName !== fragment.attribs.name)
                        throw new errors_1.PuzzleError(errors_1.ERROR_CODES.MULTIPLE_PRIMARY_FRAGMENTS);
                    primaryName = fragment.attribs.name;
                    this.fragments[fragment.attribs.name].primary = true;
                    this.fragments[fragment.attribs.name].shouldWait = true;
                }
            }
            if (!this.fragments[fragment.attribs.name].shouldWait) {
                this.fragments[fragment.attribs.name].shouldWait = typeof fragment.attribs.shouldwait !== 'undefined' || (fragment.parent && fragment.parent.name === 'head') || false;
            }
            return dependencyList;
        }, {
            gateways: {},
            fragments: {}
        });
    }
    //todo fragmentConfigleri versiyon bilgileriyle inmis olmali ki assetleri versionlara gore compile edebilelim. ayni not gatewayde de var.
    /**
     * Compiles template and returns a function that can handle the request.
     * @param {ICookieMap} testCookies
     * @param {boolean} isDebug
     * @returns {Promise<IFragmentEndpointHandler>}
     */
    compile(testCookies, isDebug = false, precompile = false) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.info(`[Compiling Page ${this.name}]`, 'Creating virtual dom');
            this.load();
            if (Object.keys(this.fragments).length === 0) {
                logger.info(`[Compiling Page ${this.name}]`, 'No fragments detected, implementing single flush handler');
                this.replaceEmptyTags();
                const singleFlushHandlerWithoutFragments = templateCompiler_1.TemplateCompiler.compile(Template.clearHtmlContent(this.dom.html()));
                return this.buildHandler(singleFlushHandlerWithoutFragments, [], [], [], isDebug);
            }
            const resourceInjector = new resource_injector_1.default(this.fragments, this.name, testCookies);
            const chunkedFragmentsWithShouldWait = Object.values(this.fragments).filter(fragment => fragment.config && fragment.shouldWait && !fragment.config.render.static);
            const chunkedFragmentsWithoutWait = Object.values(this.fragments).filter(fragment => fragment.config && !fragment.shouldWait && !fragment.config.render.static);
            const staticFragments = Object.values(this.fragments).filter(fragment => fragment.config && fragment.config.render.static);
            logger.info(`[Compiling Page ${this.name}]`, 'Injecting Puzzle Lib to head');
            resourceInjector.injectAssets(this.dom);
            resourceInjector.injectLibraryConfig(this.dom);
            // todo kaldir lib bagla
            const replaceScripts = [];
            logger.info(`[Compiling Page ${this.name}]`, 'Adding containers for waited fragments');
            const waitedFragmentReplacements = this.replaceWaitedFragmentContainers(chunkedFragmentsWithShouldWait, replaceScripts, isDebug);
            logger.info(`[Compiling Page ${this.name}]`, 'Adding containers for chunked fragments');
            const chunkReplacements = this.replaceChunkedFragmentContainers(chunkedFragmentsWithoutWait);
            this.replaceUnfetchedFragments(Object.values(this.fragments).filter(fragment => !fragment.config));
            // todo kaldir lib bag
            //await this.addDependencies();
            logger.info(`[Compiling Page ${this.name}]`, 'Replacing static contents with their real contents');
            yield this.replaceStaticFragments(staticFragments, replaceScripts.filter(replaceSet => replaceSet.fragment.config && replaceSet.fragment.config.render.static));
            logger.info(`[Compiling Page ${this.name}]`, 'Adding placeholders for chunked fragments');
            yield this.appendPlaceholders(chunkReplacements);
            logger.info(`[Compiling Page ${this.name}]`, 'Combining and minifying page styles');
            /**
             * @deprecated Combine this with only on render start assets.
             */
            yield resourceInjector.injectStyleSheets(this.dom, precompile);
            this.replaceEmptyTags();
            /**
             * todo Bu kafa olmaz runtimeda debug not debug degismez, handler ici runtime guzel olur.
             */
            const puzzleLib = fs_1.default.readFileSync(path_1.default.join(__dirname, `/lib/${isDebug ? 'puzzle_debug.min.js' : 'puzzle.min.js'}`)).toString();
            const clearLibOutput = Template.replaceCustomScripts(this.dom.html().replace('{puzzleLibContent}', puzzleLib), false);
            logger.info(`[Compiling Page ${this.name}]`, 'Sending virtual dom to compiler');
            return this.buildHandler(templateCompiler_1.TemplateCompiler.compile(Template.clearHtmlContent(clearLibOutput)), chunkReplacements, waitedFragmentReplacements, replaceScripts, isDebug);
        });
    }
    /**
     * Bind user class to page
     */
    bindPageClass() {
        const scriptMatch = templateCompiler_1.TemplateCompiler.PAGE_CLASS_CONTENT_REGEX.exec(this.rawHtml);
        if (scriptMatch) {
            const pageClass = eval(scriptMatch[1]);
            pageClass.__proto__ = new templateClass_1.TemplateClass();
            this.pageClass = pageClass;
        }
    }
    /**
     * Appends placeholders to reserved locations
     * @param {IChunkedReplacementSet[]} chunkedReplacements
     * @returns {Promise<void>}
     */
    appendPlaceholders(chunkedReplacements) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const replacement of chunkedReplacements) {
                const placeholders = replacement.replaceItems.filter(item => item.type === enums_1.REPLACE_ITEM_TYPE.PLACEHOLDER);
                for (const placeholderReplacement of placeholders) {
                    const placeholderContent = yield replacement.fragment.getPlaceholder();
                    this.dom(`[puzzle-placeholder="${placeholderReplacement.key}"]`).append(placeholderContent);
                }
            }
        });
    }
    /**
     * Replaces static fragments with their content on vDOM
     * @param {FragmentStorefront[]} fragments
     * @param {IReplaceAsset[]} replaceAssets
     * @returns {Promise<void>}
     */
    replaceStaticFragments(fragments, replaceAssets) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const fragment of fragments) {
                const partialElements = [];
                let mainElement = null;
                this.dom(`fragment[name="${fragment.name}"][from="${fragment.from}"]`).each((i, element) => {
                    if (this.dom(element).attr('partial') && this.dom(element).attr('partial') !== "main") {
                        partialElements.push(element);
                    }
                    else {
                        mainElement = element;
                    }
                });
                const assets = replaceAssets.find(set => set.fragment.name === fragment.name);
                const fragmentScripts = assets ? assets.replaceItems.reduce((script, replaceItem) => {
                    script += resource_injector_1.default.wrapJsAsset(replaceItem);
                    return script;
                }, '') : '';
                const processedAttributes = templateCompiler_1.TemplateCompiler.processExpression(mainElement.attribs, this.pageClass);
                const fragmentContent = yield fragment.getContent(processedAttributes);
                this.dom(mainElement).replaceWith(`<div id="${fragment.name}" puzzle-fragment="${fragment.name}" puzzle-gateway="${fragment.from}" fragment-partial="${'main'}">${fragmentContent.html['main'] || config_1.CONTENT_NOT_FOUND_ERROR}</div>${fragmentScripts}<script>PuzzleJs.emit('${enums_2.EVENT.ON_FRAGMENT_RENDERED}','${fragment.name}');</script>`);
                partialElements.forEach((i, element) => {
                    this.dom(element).replaceWith(`<div id="${fragment.name}" puzzle-fragment="${fragment.name}" puzzle-gateway="${fragment.from}" fragment-partial="${element.attribs.partial}">${fragmentContent.html[element.attribs.partial] || config_1.CONTENT_NOT_FOUND_ERROR}</div>${fragmentScripts}<script>PuzzleJs.emit('${enums_2.EVENT.ON_FRAGMENT_RENDERED}','${fragment.name}');</script>`);
                });
            }
        });
    }
    replaceEmptyTags() {
        this.dom('*').each((i, element) => {
            if (config_1.NON_SELF_CLOSING_TAGS.indexOf(this.dom(element)[0].name) !== -1 && this.dom(element).text().length === 0 && !this.dom(element).html()) {
                this.dom(element).text(' ');
            }
        });
    }
    /**
     * Replaces waited fragments with their content on first flush string.
     * @param {IReplaceSet[]} waitedFragments
     * @param {string} template
     * @param req
     * @param isDebug
     * @returns {Promise<IWaitedResponseFirstFlush>}
     */
    replaceWaitedFragments(waitedFragments, template, req, isDebug) {
        return __awaiter(this, void 0, void 0, function* () {
            let statusCode = enums_1.HTTP_STATUS_CODE.OK;
            let headers = {};
            let cookies = {};
            yield Promise.all(waitedFragments.map((waitedFragmentReplacement) => __awaiter(this, void 0, void 0, function* () {
                const fragmentContent = yield waitedFragmentReplacement.fragment.getContent(templateCompiler_1.TemplateCompiler.processExpression(waitedFragmentReplacement.fragmentAttributes, this.pageClass, req), req);
                if (waitedFragmentReplacement.fragment.primary) {
                    statusCode = fragmentContent.status;
                    headers = fragmentContent.headers;
                    cookies = fragmentContent.cookies;
                }
                waitedFragmentReplacement.replaceItems
                    .forEach(replaceItem => {
                    if (replaceItem.type === enums_1.REPLACE_ITEM_TYPE.CONTENT) {
                        const fragmentInject = fragmentContent.html[replaceItem.partial] || config_1.CONTENT_NOT_FOUND_ERROR;
                        template = template.replace(replaceItem.key, () => fragmentInject + Template.fragmentModelScript(waitedFragmentReplacement.fragment, fragmentContent.model, isDebug));
                    }
                });
            })));
            return { template, statusCode, headers, cookies };
        });
    }
    /**
     * Maps fragment response variable model into PuzzleLib
     * @param {{name: string}} fragment
     * @param {FragmentModel} fragmentPageModel
     * @param {boolean} isDebug
     * @returns {string}
     */
    static fragmentModelScript(fragment, fragmentPageModel, isDebug = false) {
        return fragmentPageModel && Object.keys(fragmentPageModel).length ? `<script>${Object.keys(fragmentPageModel).reduce((modelVariable, key) => {
            modelVariable += `PuzzleJs.emit("${enums_2.EVENT.ON_VARIABLES}", "${fragment.name}", "${key}", ${JSON.stringify(fragmentPageModel[key])});`;
            return modelVariable;
        }, '')}</script>` : '';
    }
    /**
     * Handles non chunked page renders
     * @param firstFlushHandler
     * @param waitedFragments
     * @param isDebug
     * @param req
     * @param res
     */
    nonChunkedHandler(firstFlushHandler, waitedFragments, isDebug, req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            this.pageClass._onRequest(req);
            const fragmentedHtml = firstFlushHandler.call(this.pageClass, req);
            const waitedReplacement = yield this.replaceWaitedFragments(waitedFragments, fragmentedHtml, req, isDebug);
            for (const prop in waitedReplacement.headers) {
                res.set(prop, waitedReplacement.headers[prop]);
            }
            for (const prop in waitedReplacement.cookies) {
                if (waitedReplacement.cookies[prop].options && waitedReplacement.cookies[prop].options.expires) {
                    waitedReplacement.cookies[prop].options.expires = new Date(String(waitedReplacement.cookies[prop].options.expires));
                }
                res.cookie(prop, waitedReplacement.cookies[prop].value, waitedReplacement.cookies[prop].options);
            }
            res.status(waitedReplacement.statusCode);
            if (waitedReplacement.statusCode === enums_1.HTTP_STATUS_CODE.MOVED_PERMANENTLY) {
                res.end();
                this.pageClass._onResponseEnd();
            }
            else {
                res.send(waitedReplacement.template.replace('</body>', () => `<script>PuzzleJs.emit('${enums_2.EVENT.ON_PAGE_LOAD}');</script></body>`));
                this.pageClass._onResponseEnd();
            }
        });
    }
    /**
     * Handles chunked page renders
     * @param firstFlushHandler
     * @param waitedFragments
     * @param chunkedFragmentReplacements
     * @param jsReplacements
     * @param isDebug
     * @param req
     * @param res
     */
    chunkedHandler(firstFlushHandler, waitedFragments, chunkedFragmentReplacements, jsReplacements, isDebug, req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            this.pageClass._onRequest(req);
            const fragmentedHtml = firstFlushHandler.call(this.pageClass, req).replace('</body>', '').replace('</html>', '');
            res.set('transfer-encoding', 'chunked');
            res.set('content-type', 'text/html; charset=UTF-8');
            const waitedPromises = [];
            //Fire requests in parallel
            const waitedReplacementPromise = this.replaceWaitedFragments(waitedFragments, fragmentedHtml, req, isDebug);
            for (let i = 0, len = chunkedFragmentReplacements.length; i < len; i++) {
                waitedPromises.push(chunkedFragmentReplacements[i].fragment.getContent(templateCompiler_1.TemplateCompiler.processExpression(chunkedFragmentReplacements[i].fragmentAttributes, this.pageClass, req), req));
            }
            //Wait for first flush
            const waitedReplacement = yield waitedReplacementPromise;
            res.set(waitedReplacement.headers);
            for (const prop in waitedReplacement.cookies) {
                res.cookie(prop, waitedReplacement.cookies[prop].value, waitedReplacement.cookies[prop].options);
            }
            res.status(waitedReplacement.statusCode);
            if (waitedReplacement.statusCode === enums_1.HTTP_STATUS_CODE.MOVED_PERMANENTLY) {
                res.end();
                this.pageClass._onResponseEnd();
            }
            else {
                res.write(waitedReplacement.template);
                res.flush();
                //Bind flush method to resolved or being resolved promises of chunked replacements
                for (let i = 0, len = chunkedFragmentReplacements.length; i < len; i++) {
                    waitedPromises[i].then(this.flush(chunkedFragmentReplacements[i], jsReplacements, res, isDebug));
                }
                //Close stream after all chunked fragments done
                yield Promise.all(waitedPromises);
                res.end(`<script>PuzzleJs.emit('${enums_2.EVENT.ON_PAGE_LOAD}');</script></body></html>`);
                this.pageClass._onResponseEnd();
            }
        });
    }
    ;
    /**
     * Creates a request handler from the compilation output. Express requests uses the return of this method
     * @param {Function} firstFlushHandler
     * @param {IReplaceSet[]} chunkedFragmentReplacements
     * @param {IReplaceSet[]} waitedFragments
     * @param {IReplaceAsset[]} jsReplacements
     * @param isDebug
     * @returns {(req: any, res: any) => void}
     */
    buildHandler(firstFlushHandler, chunkedFragmentReplacements, waitedFragments = [], jsReplacements = [], isDebug) {
        if (chunkedFragmentReplacements.length === 0) {
            return this.nonChunkedHandler.bind(this, firstFlushHandler, waitedFragments, isDebug);
        }
        else {
            // @ts-ignore https://stackoverflow.com/questions/57072381/typescript-bind-gives-type-error-when-using-more-than-4-parameters/57072988
            return this.chunkedHandler.bind(this, firstFlushHandler, waitedFragments, chunkedFragmentReplacements, jsReplacements, isDebug);
        }
    }
    /**
     * Flushes incoming fragment response
     * @param {IReplaceSet} chunkedReplacement
     * @param {IReplaceAsset[]} jsReplacements
     * @param res
     * @param isDebug
     * @returns {(fragmentContent: IFragmentContentResponse) => void}
     */
    flush(chunkedReplacement, jsReplacements, res, isDebug) {
        return (fragmentContent) => {
            const fragmentJsReplacements = jsReplacements.find(jsReplacement => jsReplacement.fragment.name === chunkedReplacement.fragment.name);
            const selfReplacing = chunkedReplacement.fragment.config && chunkedReplacement.fragment.config.render.selfReplace;
            let output = '';
            output += Template.fragmentModelScript(chunkedReplacement.fragment, fragmentContent.model, isDebug);
            fragmentJsReplacements && fragmentJsReplacements.replaceItems.filter(item => item.location === enums_1.RESOURCE_LOCATION.CONTENT_START).forEach(replaceItem => {
                output += resource_injector_1.default.wrapJsAsset(replaceItem);
            });
            chunkedReplacement.replaceItems
                .forEach(replaceItem => {
                if (replaceItem.type === enums_1.REPLACE_ITEM_TYPE.CHUNKED_CONTENT) {
                    output += `<div style="display: none;" puzzle-fragment="${chunkedReplacement.fragment.name}" puzzle-chunk-key="${replaceItem.key}">${fragmentContent.html[replaceItem.partial] || config_1.CONTENT_NOT_FOUND_ERROR}</div>`;
                    if (!(replaceItem.key === 'main' && selfReplacing)) {
                        // todo replace here
                        output += `<script>PuzzleJs.emit('${enums_2.EVENT.ON_FRAGMENT_RENDERED}','${chunkedReplacement.fragment.name}','[puzzle-chunk="${replaceItem.key}"]','[puzzle-chunk-key="${replaceItem.key}"]');</script>`;
                    }
                }
            });
            fragmentJsReplacements && fragmentJsReplacements.replaceItems.filter(item => item.location === enums_1.RESOURCE_LOCATION.CONTENT_END).forEach(replaceItem => {
                output += resource_injector_1.default.wrapJsAsset(replaceItem);
            });
            this.pageClass._onChunk(output);
            res.write(output);
            res.flush();
        };
    }
    /**
     * Clears html content from empty spaces
     * @param {string} str
     * @returns {string}
     */
    static clearHtmlContent(str) {
        return str.replace(/>\s+</g, "><").trim();
    }
    /**
     * Replaces unfetched fragments with empty div error
     * @param {FragmentStorefront[]} fragments
     */
    replaceUnfetchedFragments(fragments) {
        fragments.forEach(fragment => {
            this.dom(`fragment[from="${fragment.from}"][name="${fragment.name}"]`).replaceWith(`<div puzzle-fragment="${fragment.name}" puzzle-gateway="${fragment.from}">${config_1.CONTENT_NOT_FOUND_ERROR}</div>`);
        });
    }
    /**
     * Creates chunked fragment containers
     * @param {FragmentStorefront[]} chunkedFragments
     * @returns {IReplaceSet[]}
     */
    replaceChunkedFragmentContainers(chunkedFragments) {
        const chunkReplacements = [];
        chunkedFragments.forEach(fragment => {
            const replaceItems = [];
            let fragmentAttributes = {};
            this.dom(`fragment[from="${fragment.from}"][name="${fragment.name}"]`)
                .each((i, element) => {
                const partial = element.attribs.partial || 'main';
                const contentKey = fragment.name + '_' + partial;
                const replaceItem = {
                    type: enums_1.REPLACE_ITEM_TYPE.CHUNKED_CONTENT,
                    partial,
                    key: contentKey,
                };
                if (partial === 'main') {
                    fragmentAttributes = element.attribs;
                }
                replaceItems.push(replaceItem);
                if (fragment.config && fragment.config.render.placeholder && replaceItem.partial === 'main') {
                    const placeholderContentKey = contentKey + '_placeholder';
                    replaceItems.push({
                        type: enums_1.REPLACE_ITEM_TYPE.PLACEHOLDER,
                        partial,
                        key: placeholderContentKey
                    });
                    this.dom(element).replaceWith(`<div id="${fragment.name}" puzzle-fragment="${element.attribs.name}" puzzle-gateway="${element.attribs.from}" ${element.attribs.partial ? 'fragment-partial="' + element.attribs.partial + '"' : ''} puzzle-chunk="${contentKey}" puzzle-placeholder="${placeholderContentKey}"></div>`);
                }
                else {
                    this.dom(element).replaceWith(`<div id="${fragment.name}" puzzle-fragment="${element.attribs.name}" puzzle-gateway="${element.attribs.from}" ${element.attribs.partial ? 'fragment-partial="' + element.attribs.partial + '"' : ''} puzzle-chunk="${contentKey}"> </div>`);
                }
            });
            chunkReplacements.push({
                fragment,
                replaceItems,
                fragmentAttributes
            });
        });
        return chunkReplacements;
    }
    /**
     * Creates containers for fragments should be waited
     * @param {FragmentStorefront[]} fragmentsShouldBeWaited
     * @param {IReplaceAsset[]} replaceJsAssets
     * @returns {IReplaceSet[]}
     */
    replaceWaitedFragmentContainers(fragmentsShouldBeWaited, replaceJsAssets, isDebug) {
        const waitedFragmentReplacements = [];
        fragmentsShouldBeWaited.forEach(fragment => {
            const replaceItems = [];
            let fragmentAttributes = {};
            const jsReplacements = replaceJsAssets.find(jsReplacement => jsReplacement.fragment.name === fragment.name);
            let contentStart = '';
            let contentEnd = ``;
            jsReplacements && jsReplacements.replaceItems.filter(item => item.location === enums_1.RESOURCE_LOCATION.CONTENT_START).forEach(replaceItem => {
                contentStart += resource_injector_1.default.wrapJsAsset(replaceItem);
            });
            jsReplacements && jsReplacements.replaceItems.filter(item => item.location === enums_1.RESOURCE_LOCATION.CONTENT_END).forEach(replaceItem => {
                contentEnd += resource_injector_1.default.wrapJsAsset(replaceItem);
            });
            this.dom(contentStart).insertBefore(this.dom(`fragment[from="${fragment.from}"][name="${fragment.name}"]`).first());
            this.dom(contentEnd).insertAfter(this.dom(`fragment[from="${fragment.from}"][name="${fragment.name}"]`).last());
            this.dom(`fragment[from="${fragment.from}"][name="${fragment.name}"]`)
                .each((i, element) => {
                const replaceKey = `{fragment|${element.attribs.name}_${element.attribs.from}_${element.attribs.partial || 'main'}}`;
                const partial = element.attribs.partial || 'main';
                replaceItems.push({
                    type: enums_1.REPLACE_ITEM_TYPE.CONTENT,
                    key: replaceKey,
                    partial,
                });
                if (partial === 'main') {
                    fragmentAttributes = element.attribs;
                }
                if (element.parentNode.name !== 'head') {
                    this.dom(element).replaceWith(`<div id="${fragment.name}" puzzle-fragment="${element.attribs.name}" puzzle-gateway="${element.attribs.from}" ${element.attribs.partial ? 'fragment-partial="' + element.attribs.partial + '"' : ''}>${replaceKey}</div><script>PuzzleJs.emit('${enums_2.EVENT.ON_FRAGMENT_RENDERED}','${fragment.name}');</script>`);
                }
                else {
                    this.dom(element).replaceWith(replaceKey);
                }
            });
            waitedFragmentReplacements.push({
                fragment,
                replaceItems,
                fragmentAttributes
            });
        });
        return waitedFragmentReplacements;
    }
    static replaceCustomScripts(template, encode) {
        return encode ?
            template.replace(/<puzzle-script>/g, '<!--custom-script').replace(/<\/puzzle-script>/g, '/custom-script-->') :
            template.replace(/<!--custom-script/g, '<script>').replace(/\/custom-script-->/g, '</script>');
    }
}
__decorate([
    decorators_1.benchmark(util_1.isDebug(), logger.info),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], Template.prototype, "getDependencies", null);
__decorate([
    decorators_1.benchmark(util_1.isDebug(), logger.info),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", void 0)
], Template.prototype, "replaceUnfetchedFragments", null);
__decorate([
    decorators_1.benchmark(util_1.isDebug(), logger.info),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", void 0)
], Template.prototype, "replaceChunkedFragmentContainers", null);
__decorate([
    decorators_1.benchmark(util_1.isDebug(), logger.info),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array, Array, Boolean]),
    __metadata("design:returntype", void 0)
], Template.prototype, "replaceWaitedFragmentContainers", null);
exports.Template = Template;
//# sourceMappingURL=template.js.map