"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("./errors");
class TemplateCompiler {
    /**
     * Process object values and convert it using binding object.
     * @param expression
     * @param binding
     * @param req
     * */
    static processExpression(expression, binding, req) {
        const processedExpression = Object.assign({}, expression);
        Object.keys(processedExpression).forEach((key) => {
            if (this.isExpression(processedExpression[key])) {
                const expressionVariable = processedExpression[key].split('${')[1].split('}')[0];
                processedExpression[key] = (new Function("req", `return ${expressionVariable}`)).apply(binding, [req]);
            }
        });
        return processedExpression;
    }
    /**
     * Checks if there is any string interpolation.
     * @param {string} template
     * @returns {boolean}
     */
    static isExpression(template) {
        return template.indexOf('${') > -1;
    }
    /**
     * Compiles given html string into function that acceps req as an argument.
     * @param {string} template
     * @returns {Function}
     */
    static compile(template) {
        if (!this.isExpression(template))
            return () => template;
        let generatedFn = `let out = '';`;
        const partials = template.split(this.TEMPLATE_REGEX);
        for (let x = 0, len = partials.length; x < len; x++) {
            if (this.isExpression(partials[x])) {
                const expressionMathes = partials[x].match(this.EXPRESSION_REGEX);
                if (expressionMathes) {
                    const expression = expressionMathes[1];
                    if (expression.match(/if|for|else|switch|case|break|{|}/)) {
                        generatedFn += expression;
                    }
                    else {
                        generatedFn += 'out+=\`${' + expression + '}\`;';
                    }
                }
            }
            else {
                generatedFn += `out+=\`${partials[x]}\`;`;
            }
        }
        try {
            return new Function('req', `${generatedFn}return out;`);
        }
        catch (e) {
            throw new errors_1.PuzzleError(errors_1.ERROR_CODES.FAILED_TO_COMPILE_TEMPLATE, `${generatedFn}return out;`);
        }
    }
}
TemplateCompiler.TEMPLATE_REGEX = /(\${.*?}?})/;
TemplateCompiler.EXPRESSION_REGEX = /^\${(.*?)}$/;
TemplateCompiler.TEMPLATE_CONTENT_REGEX = /<template>(.*?)<\/template>/mis;
TemplateCompiler.PAGE_CLASS_CONTENT_REGEX = /<script>(.*?)<\/script>(.*)<template>/mis;
exports.TemplateCompiler = TemplateCompiler;
//# sourceMappingURL=templateCompiler.js.map